local ENABLED = nil;

function HealPointsFrame_OnLoad()
	local _, className = UnitClass("player");
	if (className == "PALADIN" or className == "PRIEST" or className == "DRUID" or className == "SHAMAN") then
		ENABLED = 1;
	end

	if (ENABLED) then
		-- Register events
		this:RegisterEvent("UNIT_AURA");
		this:RegisterEvent("UNIT_LEVEL");
		this:RegisterEvent("UNIT_MAXMANA");
		this:RegisterEvent("UNIT_STATS");
		this:RegisterEvent("UNIT_INVENTORY_CHANGED");
		this:RegisterEvent("LEARNED_SPELL_IN_TAB");
		this:RegisterEvent("CHARACTER_POINTS_CHANGED");
	
		DEFAULT_CHAT_FRAME:AddMessage("HealPoints loaded.",1,1,1);
	else
		DEFAULT_CHAT_FRAME:AddMessage("HealPoints not loaded - not a healing class",1,1,1);
	end
end

function HealPointsFrame_OnEvent(event)
	if (not CharacterFrame:IsVisible() or ENABLED == nil) then
		return;
	end
	-- Do nothing if event target is not player
	if ((	event == "UNIT_AURA" or 
			event == "UNIT_LEVEL" or 
			event == "UNIT_MAXMANA" or 
			event == "UNIT_STATS" or 
			event == "UNIT_INVENTORY_CHANGED") 
			and not UnitIsUnit(arg1, "player")) then
		return;
	end	
	HealPointsFrame_SetPoints();
end

function HealPointsFrame_OnEnter()
	if (ENABLED) then
		GameTooltip:SetOwner(this, "ANCHOR_RIGHT");
		GameTooltip:SetText("HealPoints "..format("%5.0f", this.healPoints),
								HIGHLIGHT_FONT_COLOR.r, HIGHLIGHT_FONT_COLOR.g, HIGHLIGHT_FONT_COLOR.b);
		GameTooltip:AddDoubleLine("Mana regeneration while casting/5s:",
										format("%5.0f", this.manaRegenCasting),
										NORMAL_FONT_COLOR.r, NORMAL_FONT_COLOR.g, NORMAL_FONT_COLOR.b,
										HIGHLIGHT_FONT_COLOR.r, HIGHLIGHT_FONT_COLOR.g, HIGHLIGHT_FONT_COLOR.b);
		GameTooltip:AddDoubleLine("Normal mana regeneration/5s:",
										format("%5.0f", this.manaRegenNormal),
										NORMAL_FONT_COLOR.r, NORMAL_FONT_COLOR.g, NORMAL_FONT_COLOR.b,
										HIGHLIGHT_FONT_COLOR.r, HIGHLIGHT_FONT_COLOR.g, HIGHLIGHT_FONT_COLOR.b);
		GameTooltip:AddDoubleLine("+Healing from items:",
										format("%5.0f", this.healing),
										NORMAL_FONT_COLOR.r, NORMAL_FONT_COLOR.g, NORMAL_FONT_COLOR.b,
										HIGHLIGHT_FONT_COLOR.r, HIGHLIGHT_FONT_COLOR.g, HIGHLIGHT_FONT_COLOR.b);
		GameTooltip:AddDoubleLine("Chance to crit with healing spells:",
										format("%.2f%%", this.spellCrit),
										NORMAL_FONT_COLOR.r, NORMAL_FONT_COLOR.g, NORMAL_FONT_COLOR.b,
										HIGHLIGHT_FONT_COLOR.r, HIGHLIGHT_FONT_COLOR.g, HIGHLIGHT_FONT_COLOR.b);
		GameTooltip:AddLine("Hint: Click to show the HealPoints calculator",
								GREEN_FONT_COLOR.r, GREEN_FONT_COLOR.g, GREEN_FONT_COLOR.b);
		GameTooltip:Show();
	end
end

function HealPointsFrame_OnShow()
	if (ENABLED) then
		HealPointsFrame_SetPoints();
	end
end

function HealPointsFrame_OnClick()
	if (HealPointsCalculator:IsVisible()) then
		HealPointsCalculator:Hide();
	elseif (ENABLED) then
		HealPointsCalculator:Show();
	end
end

function HealPointsFrame_SetPoints()
	local unit = "player";
	local label = getglobal("HealPointsFrameLabel");
	local text = getglobal("HealPointsFrameStatText");
	local frame = getglobal("HealPointsFrame");

	local manaRegenCasting, manaRegenNormal = HealPointsCalculator_ComputeManaRegen();
	local healing = HealPointsBS_GetBonus('HEAL');
	local spellCrit = HealPointsCalculator_ComputeSpellCrit();
	local mana = UnitManaMax(unit);
	local healPoints = HealPointsCalculator_ComputeHealpoints(healing, spellCrit, mana, manaRegenCasting, manaRegenNormal);
	
	frame.healPoints = healPoints;
	frame.manaRegenCasting = manaRegenCasting;
	frame.manaRegenNormal = manaRegenNormal;
	frame.healing = healing;
	frame.spellCrit = spellCrit;

	label:SetText("HealPoints:");
	text:SetText(format("%5.0f", healPoints));
end
