HEALPONTSBS_PATTERN_SETNAME = "^(.*) %(%d/%d%)$";
HEALPONTSBS_PATTERN_GENERIC_PREFIX = "^%+(%d+)%%?(.*)$";
HEALPONTSBS_PATTERN_GENERIC_SUFFIX = "^(.*)%+(%d+)%%?$";

HealPointsBS_bonuses = {};
HealPointsBS_bonuses_details = {};
HealPointsBS_active = nil;
HealPointsBS_temp = { 
	sets = {},
	set = "",
	slot = "",
	bonuses = {},
	details = {}
}

HealPointsBS_types = {
	"STR", 		-- strength	
	"AGI", 		-- agility
	"STA", 		-- stamina
	"INT", 		-- intellect
	"SPI", 		-- spirit
	"ARMOR", 	-- reinforced armor (not base armor)
  
	"ARCANERES",-- arcane resistance
	"FIRERES",  -- fire resistance
	"NATURERES",-- nature resistance 	
	"FROSTRES", -- frost resistance
	"SHADOWRES",-- shadow resistance

	"FISHING",  -- fishing skill
	"MINING",	-- mining skill
	"HERBALISM",-- herbalism skill
	"SKINNING", -- skinning skill
	"DEFENSE",  -- defense skill

	"BLOCK",    -- chance to block
	"DODGE",	-- chance to dodge
	"PARRY",	-- chance to parry
	"ATTACKPOWER", -- attack power
	"CRIT",		-- chance to get a critical strike
	"RANGEDATTACKPOWER", -- ranged attack power
	"RANGEDCRIT", -- chance to get a crit with ranged weapons
	"TOHIT",	-- chance to hit

	"DMG",		-- spell damage
	"ARCANEDMG",-- arcane spell damage
	"FIREDMG",	-- fire spell damage
	"FROSTDMG",	-- frost spell damage
	"HOLYDMG",	-- holy spell damage
	"NATUREDMG",-- nature spell damage
	"SHADOWDMG",-- shadow spell damage
	"SPELLCRIT",-- chance to crit with spells
	"HEAL",		-- healing 
	"HOLYCRIT", -- chance to crit with holy spells

	"HEALTHREG",-- health regeneration per 5 sec.
	"MANAREG",	-- mana regeneration per 5 sec.
	"HEALTH",	-- health points
	"MANA",		-- mana points

	-- Added
	"CASTINGREG", 		-- % mana regeneratio while casting 			
	"NATURECRIT", 		-- chance to crit with nature spells      
	"CASTINGREGROWTH", 	-- reduced casting time Regrowth     
	"CASTINGHOLYLIGHT", -- reduced casting time Holy Light  
	"CASTINGFLASHHEAL", -- reduced casting time Flash Heal  
	"DURATIONREJUV"  	-- increased duration Rejuvenation  
}

HealPointsBS_slots = {
	"Head",
	"Neck",
	"Shoulder",
	"Shirt",
	"Chest",
	"Waist",
	"Legs",
	"Feet",
	"Wrist",
	"Hands",
	"Finger0",
	"Finger1",
	"Trinket0",
	"Trinket1",
	"Back",
	"MainHand",
	"SecondaryHand",
	"Ranged",
	"Tabard",
};

-- Local functions
local function addValue(effect, value)	
	if(type(effect) == "string") then
		if(HealPointsBS_temp.bonuses[effect]) then
			HealPointsBS_temp.bonuses[effect] = HealPointsBS_temp.bonuses[effect] + value;
		else
			HealPointsBS_temp.bonuses[effect] = value;
		end
		
		if(HealPointsBS_temp.slot) then
			if(HealPointsBS_temp.details[effect]) then
				if(HealPointsBS_temp.details[effect][HealPointsBS_temp.slot]) then
					HealPointsBS_temp.details[effect][HealPointsBS_temp.slot] = HealPointsBS_temp.details[effect][HealPointsBS_temp.slot] + value;
				else
					HealPointsBS_temp.details[effect][HealPointsBS_temp.slot] = value;
				end
			else
				HealPointsBS_temp.details[effect] = {};
				HealPointsBS_temp.details[effect][HealPointsBS_temp.slot] = value;
			end
		end
		else 
		-- list of effects
		local i,e;
		for i,e in effect do
			addValue(e, value);
		end
	end
end;

local function checkToken(token, value)
	-- Identifies simple tokens like "Intellect" and composite tokens like "Fire damage" and 
	-- add the value to the respective bonus. 
	-- returns true if some bonus is found	
	if(HEALPONTSBS_PATTERNS_GENERIC_LOOKUP[token]) then
		addValue(HEALPONTSBS_PATTERNS_GENERIC_LOOKUP[token], value);
		return true;
	end
	
	local s1 = nil;
	local s2 = nil;
	local i, p;
	for i,p in HEALPONTSBS_PATTERNS_GENERIC_STAGE1 do
		if(string.find(token,p.pattern,1,1)) then
			s1 = p.effect;
		end
	end	
	for i,p in HEALPONTSBS_PATTERNS_GENERIC_STAGE2 do
		if(string.find(token,p.pattern,1,1)) then
			s2 = p.effect;
		end
	end	
	if(s1 and s2) then
		addValue(s1..s2, value);
		return true;
	end 
	return false;
end

local function checkGeneric(line)
	local function trim(str)
	    str = string.gsub( str, "^%s+", "" );
   	    str = string.gsub( str, "%s+$", "" );
       	return string.gsub( str, "%.$", "" );
	end

	-- Scans generic bonuses like "+3 Intellect" or "Arcane Resistance +4"
	-- split line at "/" for enchants with multiple effects
	local found = false;
	local value, token, pos, tmpStr;
	while(string.len(line) > 0) do
		pos = string.find(line, "/", 1, true);
		if(pos) then
			tmpStr = string.sub(line,1,pos-1);
			line = string.sub(line,pos+1);
		else
			tmpStr = line;
			line = "";
		end
		
		tmpStr = trim(tmpStr); -- trim line

		_, _, value, token = string.find(tmpStr, HEALPONTSBS_PATTERN_GENERIC_PREFIX);
		if(not value) then
			_, _, token, value = string.find(tmpStr, HEALPONTSBS_PATTERN_GENERIC_SUFFIX);
		end
		if(token and value) then			
			token = trim(token); -- trim token
	
			if(checkToken(token,value)) then
				found = true;
			end
		end
	end
	return found;
end

local function checkPassive(line)
	-- Scans passive bonuses like "Set: " and "Equip: "
	local found = nil;
	local i, p, value;
	for i,p in HEALPONTSBS_PATTERNS_PASSIVE do
		_, _, value = string.find(line, "^" .. p.pattern);
		if(value) then
			addValue(p.effect, value)
			found = 1;
		end
	end
	if(not found) then
		checkGeneric(line);
	end
end

local function checkOther(line)
	-- Last fallback for not generic enchants, like "Mana Regen x per 5 sec."
	local i, p, value, start;
	for i,p in HEALPONTSBS_PATTERNS_OTHER do
		start, _, value = string.find(line, "^" .. p.pattern);
		if(start) then
			if(p.value) then
				addValue(p.effect, p.value)
			elseif(value) then
				addValue(p.effect, value)
			end
			return true;
		end
	end
	return false;
end

local function scanLine(line)	
	-- Check for "Equip: "
	if(string.sub(line,0,string.len(HEALPONTSBS_PREFIX_EQUIP)) == HEALPONTSBS_PREFIX_EQUIP) then

		local tmpStr = string.sub(line,string.len(HEALPONTSBS_PREFIX_EQUIP)+1);
		checkPassive(tmpStr);

	-- Check for "Set: "
	elseif(string.sub(line,0,string.len(HEALPONTSBS_PREFIX_SET)) == HEALPONTSBS_PREFIX_SET
			and HealPointsBS_temp.set ~= "" 
			and not HealPointsBS_temp.sets[HealPointsBS_temp.set]) then

		local tmpStr = string.sub(line,string.len(HEALPONTSBS_PREFIX_SET)+1);
		HealPointsBS_temp.slot = "Set";
		checkPassive(tmpStr);

	-- any other line (standard stats, enchantment, set name, etc.)
	else
		-- Check for set name
		local _, _, tmpStr = string.find(line, HEALPONTSBS_PATTERN_SETNAME);
		if(tmpStr) then
			HealPointsBS_temp.set = tmpStr;
		else
			local found = checkGeneric(line);
			if(not found) then
				checkOther(line);
			end
		end
	end
end

local function scanTooltip()
	local tmpTxt;
	local lines = HealPointsBSTooltip:NumLines();
	local itemName = HealPointsBSTooltipTextLeft1:GetText();
	
	for i=2, lines, 1 do
		tmpText = getglobal("HealPointsBSTooltipTextLeft"..i);
		if (tmpText:GetText()) then
			scanLine(tmpText:GetText());
		end
	end
end

local function scanEquipment()
	local slotid, slotname, hasItem, i;

	HealPointsBS_temp.bonuses = {};
	HealPointsBS_temp.details = {};
	HealPointsBS_temp.sets = {};
	HealPointsBS_temp.set = "";

	for i, slotname in HealPointsBS_slots do
		slotid, _ = GetInventorySlotInfo(slotname.. "Slot");
		hasItem = HealPointsBSTooltip:SetInventoryItem("player", slotid);
	
		if (hasItem) then
			HealPointsBS_temp.slot = slotname;
			scanTooltip();
			-- if set item, mark set as already scanned
			if(HealPointsBS_temp.set ~= "") then
				HealPointsBS_temp.sets[HealPointsBS_temp.set] = 1;
			end;
		end
	end
	HealPointsBS_bonuses = HealPointsBS_temp.bonuses;
	HealPointsBS_bonuses_details = HealPointsBS_temp.details;
end

-- Global functions
function HealPointsBS_Update()
	-- Update function to hook into. 
	-- Gets called, when Equipment changes (after UNIT_INVENTORY_CHANGED)
end

function HealPointsBS_GetBonus(bonus)
	if(HealPointsBS_bonuses[bonus]) then
		if (type(HealPointsBS_bonuses[bonus]) == "string") then
			return tonumber(HealPointsBS_bonuses[bonus]);
		else
			return HealPointsBS_bonuses[bonus];
		end
	end
	return 0;
end

function HealPointsBS_OnLoad()
	this:RegisterEvent("PLAYER_ENTERING_WORLD");
	this:RegisterEvent("UNIT_INVENTORY_CHANGED");
end

function HealPointsBS_OnEvent()
	if (event == "PLAYER_ENTERING_WORLD") then
		HealPointsBS_active = 1;
	end
	if (((event == "PLAYER_ENTERING_WORLD") or (event == "UNIT_INVENTORY_CHANGED")) and HealPointsBS_active) then
		scanEquipment();
		HealPointsBS_Update();
	end
end
--